-- Make pulleys give you rope hang (object-side)
local function pulleyHangTouch(phandle, playermo)
	if not (phandle and phandle.valid and playermo
	and playermo.valid and playermo.player) then return end
	local p = playermo.player
	p.pulleylasttic = true
	p.powers[pw_carry] = CR_ROPEHANG
	p.mo.flags = $ | MF_NOCLIP
	-- What if you die mid pulley?
	if (p.pflags & PF_THOKKED) then p.pflags = $ & ~PF_THOKKED end
end

addHook("TouchSpecial", pulleyHangTouch, MT_PULLEYHANDLE)

-- Make pulleys give you rope hang (player-side)
local function pulleyHangPlayer(p)
	if p.pulleylasttic and (p.mo.state != S_PLAY_RIDE
	or p.playerstate != PST_LIVE) then
		p.pulleylasttic = false
		p.powers[pw_carry] = CR_NONE
		p.mo.flags = $ & ~MF_NOCLIP
	end
end

addHook("PlayerThink", pulleyHangPlayer)

-- Remove remnants of allchars pulleys
local function allCharsPulley(mo)
	if mo and mo.valid then
		if not (mo.base and mo.base.valid) then
			P_RemoveMobj(mo.rope)
			P_RemoveMobj(mo)
		end
	end
end

addHook("MobjThinker", allCharsPulley, MT_PULLEYHANDLE)

-- Stop pulley handles from being papersprites
local function noPaperPulley(mo)
	if mo and mo.valid then
		mo.frame = $ & ~FF_PAPERSPRITE
	end
end

addHook("MobjSpawn", noPaperPulley, MT_PULLEYHANDLE)

-- Fix dash ring functionality
local function dashRingFix(special, toucher)
    if toucher and toucher.valid
	and special and special.valid
	and toucher.player then
		local p = toucher.player
		P_ResetPlayer(p)
		p.mo.state = S_PLAY_FALL
		if (p.pflags & PF_STARTJUMP) then p.pflags = $ & ~PF_STARTJUMP end
	end
end

addHook("TouchSpecial", dashRingFix, MT_DASHRING)

-- DUMB HACK TO MAKE PULLEYS HOME-ATTACKABLE

-- Spawn the dummy crawla when a pulley spawns
local function homeAttackPulleys_pulley(mo)
    if mo and mo.valid then
		local pulleyCrawla = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_BLUECRAWLA)
		pulleyCrawla.pulleysource = mo
	end
end

addHook("MobjSpawn", homeAttackPulleys_pulley, MT_PULLEYHANDLE)

-- Make the crawla constantly follow the pulley
local function homeAttackPulleys_crawla(mo)
    if mo and mo.valid and mo.pulleysource then
		mo.flags2 = $ | MF2_DONTDRAW
		mo.flags = $ | MF_NOCLIP
		mo.scale = mo.pulleysource.scale
		if mo.pulleysource.valid then
			P_MoveOrigin(mo, mo.pulleysource.x, mo.pulleysource.y, mo.pulleysource.z)
			return true
		else
			P_RemoveMobj(mo)
		end
	end
end

addHook("MobjThinker", homeAttackPulleys_crawla, MT_BLUECRAWLA)

-- No collision for crawla
local function homeAttackPulleys_collide_crawla(thing, tmthing)
    if thing and tmthing and tmthing.pulleysource then
		return false
	end
end

addHook("MobjCollide", homeAttackPulleys_collide_crawla, MT_BLUECRAWLA)

-- No touch special for crawla
local function homeAttackPulleys_touch_crawla(special, toucher)
	if special and special.valid and special.pulleysource then
		return true
	end
end

addHook("TouchSpecial", homeAttackPulleys_touch_crawla, MT_BLUECRAWLA)

-- Player stops targeting when it reaches the pulley
local function homeAttackPulleys_touch_pulley(special, toucher)
	if special and special.valid and special.pulleysource
	and toucher and toucher.valid and toucher.player
	and toucher.player.homing then
		toucher.player.homing = 0
	end
end

addHook("TouchSpecial", homeAttackPulleys_touch_pulley, MT_BLUECRAWLA)

-- Can't damage the crawla
local function homeAttackPulleys_damage_crawla(target)
    if target and target.pulleysource then return true end
end

addHook("MobjDamage", homeAttackPulleys_damage_crawla, MT_BLUECRAWLA)
addHook("MobjDeath", homeAttackPulleys_damage_crawla, MT_BLUECRAWLA)

-- Crawla can't damage you
local function homeAttackPulleys_damage_player(target, inflictor, source)
    if target and target.player and source
	and source.type == MT_BLUECRAWLA and source.pulleysource then
		return true
	end
end

addHook("MobjDamage", homeAttackPulleys_damage_player, MT_PLAYER)
addHook("MobjDeath", homeAttackPulleys_damage_player, MT_PLAYER)